<?php

// Scanner AJAX actions
require_once(SPBC_PLUGIN_DIR . 'inc/spbc-scanner.php');

/*
 * Contactins setting page functions
 * Included from /security-malware-firewall.php -> /inc/spbc-admin.php
 */

/**
 * Action 'admin_menu' - Add the admin options page
 * 
 * @global type $spbc
 */
function spbc_admin_add_page() {
	
	// Adding setting page
	if(is_network_admin())
		add_submenu_page("settings.php", __( SPBC_NAME . ' Settings', 'security-malware-firewall'), SPBC_NAME, 'manage_options', 'spbc', 'spbc_settings_page');
	else
		add_options_page(                __( SPBC_NAME . ' Settings', 'security-malware-firewall'), SPBC_NAME, 'manage_options', 'spbc', 'spbc_settings_page');
	
	// Register setting
    register_setting(SPBC_SETTINGS, SPBC_SETTINGS, 'spbc_sanitize_settings');
		
	spbc_settings__register();
	
}

function spbc_settings__register() {
	
	global $spbc;
	
    $spbc->settings__elements = spbc_settings__register_sections_and_fields(array(
			
			// STATUS. Security status.
			'status' => array(
				'type'   => 'plain',
				'callback'    => 'spbc_field_security_status',
			),
			// BUTTONS
			// Dashboard 
			'cp_button' => array(
				'type' => 'plain',
				'display' => $spbc->key_is_ok,
				'html'    => '<div id="goToCleanTalk" class="spbc-div-2" style="display: inline-block; position: relative; top: -2px; margin-right: 7px;">'
						. '<a id="goToCleanTalkLink" class="spbc_manual_link" target="_blank" href="https://cleantalk.org/my?user_token=<?php echo $spbc->user_token ?>&cp_mode=security">'
						. __('Security Control Panel', 'security-malware-firewall')
						. '</a>'
					. '</div>',
			),
			// Support
			'support_button' => array(
				'type' => 'plain',
				'html'    => '<a target="_blank" href="https://wordpress.org/support/plugin/security-malware-firewall" style="display: inline-block; position: relative; top: -2px;">'
						. '<input type="button" class="spbc_auto_link" value="'.__('Support', 'security-malware-firewall').'" />'
					. '</a>'
					. '<br>',
			),
			
			// TABS
			// Scanner
			'scanner' => array(
				'type' => 'tab',
				'display' => is_main_site() && $spbc->scaner_enabled,
				'title' => __('Malware Scanner', 'security-malware-firewall'),
				'icon' => 'icon-search',
				'class_prefix' => 'spbc',
				'active' => true,
				'ajax' => true,
				'js_before' => 'scanner-plugin.js',
				'js_after' => 'settings_tab--scanner.js',
				'sections' => array(
					'scanner' => array(
						'type' => 'section',
						'fields' => array(
							'scanner' => array(
								'type' => 'field',
								'callback' => 'spbc_field_scaner'
							),
						),
					),
				),
			),
			// Security log
			'security_log' => array(
				'type' => 'tab',
				'title' => __('Security Log', 'security-malware-firewall'),
				'icon' => 'user-secret',
				'class_prefix' => 'spbc',
				'ajax' => true,
				'js_after' => 'settings_tab--security_log.js',
				'sections' => array(
					'security_log' => array(
						'type' => 'section',
						'fields' => array(
							'security_log' => array(
								'type' => 'field',
								'callback' => 'spbc_field_security_logs'
							),
						),
					),
				),
			),
			// Firewall
			'traffic_control' => array(
				'type' => 'tab',
				'display' => is_main_site() && $spbc->fw_enabled,
				'title' => __('Firewall', 'security-malware-firewall'),
				'icon' => 'icon-exchange',
				'class_prefix' => 'spbc',
				'ajax' => true,
				'js_after' => 'settings_tab--traffic_control.js',
				'sections' => array(
					'tc_log' => array(
						'type' => 'section',
						'fields' => array(
							'tc_log' => array(
								'type' => 'field',
								'callback' => 'spbc_field_traffic_control_log'
							),
						),
					),
				),
			),
			// Settings
			'settings_general' => array(
				'type' => 'tab',
				'title' => __('General Settings', 'security-malware-firewall'),
				'icon' => 'icon-sliders',
				'class_prefix' => 'spbc',
				'ajax' => true,
				'js_after'  => 'settings_tab--settings_general.js', 
				'after' => 'submit_button',
				'sections' => array(
					'apikey' => array(
						'type' => 'section',
						'title' => __('Access Key', 'security-malware-firewall'),
						'fields' => array(
							'apikey' => array(
								'type' => 'field',
								'callback' => 'spbc_field_key'
							),
							'custom_key' => array(
								'type' => 'field',
								'title' => __('Allow users to use other key', 'security-malware-firewall'),
								'description' => __('Allow users to use different Access key in their plugin settings. They could use different CleanTalk account.', 'security-malware-firewall'),
								'display' => is_main_site() && SPBC_WPMS,
							),
						),
					),
					'firewall' => array(
						'type' => 'section',
						'title' => __('Firewall', 'security-malware-firewall'),
						'display' => $spbc->fw_enabled,
						'fields' => array(
							'waf_enabled' => array(
								'type' => 'field',
								'title' => __('Web Application Firewall', 'security-malware-firewall'),
								'description' => __('Catches dangerous stuff like: XSS, MySQL-injections and malicious uploaded files.', 'security-malware-firewall'),
								'childrens' => array('waf_xss_check','waf_sql_check','waf_file_check'),
							),
							'waf_xss_check' => array(
								'type' => 'field',
								'title' => __('XSS check', 'security-malware-firewall'),
								'description' => __('The website\'s mirrors. Separated with comma, without protocols (examle: "some.com, example.com, my.gov").', 'security-malware-firewall'),
								'parent' => 'waf_enabled',
							),
							'waf_sql_check' => array(
								'type' => 'field',
								'title' => __('SQL-injection check', 'security-malware-firewall'),
								'description' => __('SQL-injection test.', 'security-malware-firewall'),
								'parent' => 'waf_enabled',
							),
							'waf_file_check' => array(
								'type' => 'field',
								'title' => __('Check uploaded files', 'security-malware-firewall'),
								'description' => __('Check uploaded files for malicious code.', 'security-malware-firewall'),
								'parent' => 'waf_enabled',
							),
							'traffic_control_enabled' => array(
								'type' => 'field',
								'title' => __('Traffic control', 'security-malware-firewall'),
								'description' => __('Traffic Control shows a list of visits and hits towards your website. Allows you to ban any visitor, a whole country or a network.', 'security-malware-firewall'),
								'childrens' => array('traffic_control_autoblock_amount'),
							),
							'traffic_control_autoblock_amount' => array(
								'input_type' => 'text',
								'type' => 'field',
								'title' => __('Block user after more than N requests per hour.', 'security-malware-firewall'),
								'class' => 'spbc_short_text_field',
								'parent' => 'traffic_control_enabled',
							),
						),
					),
					'scanner_setting' => array(
						'type' => 'section',
						'title' => __('Malware scanner', 'security-malware-firewall'),
						'display' => $spbc->scaner_enabled,
						'fields' => array(
							'scanner_outbound_links' => array(
								'type' => 'field',
								'title' => __('Scan links', 'security-malware-firewall'),
								'description' => __('Turning this option on may increase scanning time for websites with a lot of pages.', 'security-malware-firewall'),
								'childrens' => array('scanner_outbound_links_mirrors'),
							),
							'scanner_outbound_links_mirrors' => array(
								'type' => 'field',
								'input_type' => 'text',
								'parent' => 'scanner_outbound_links',
								'title' => __('Website\'s mirrors', 'security-malware-firewall'),
								'description' => __('The website\'s mirrors. Separated with comma, without protocols (examle: "some.com, example.com, my.gov").', 'security-malware-firewall'),
								'class' => 'spbc_long_text_field',
							),
							'scanner_heuristic_analysis' => array(
								'type' => 'field',
								'title' => __('Heuristic analysis', 'security-malware-firewall'),
								'description' => __('Will search for dangerous code in modified file.', 'security-malware-firewall'),
							),
							'scanner_signature_analysis' => array(
								'type' => 'field',
								'title' => __('Signature analysis', 'security-malware-firewall'),
								'description' => __('Will search for known malicious signatures in files.', 'security-malware-firewall'),
							),
							'scanner_auto_cure' => array(
								'type' => 'field',
								'title' => __('Cure malware', 'security-malware-firewall'),
								'description' => __('Will cure know malware.', 'security-malware-firewall'),
							),
						),
					),
					'misc' => array(
						'type' => 'section',
						'title' => __('Miscellaneous', 'security-malware-firewall'),
						'fields' => array(
							'backend_logs_enable' => array(
								'display' => is_main_site(),
								'type' => 'field',
								'title' => __('Collect and send PHP logs', 'security-malware-firewall'),
								'description' => __('Collec and send PHP logs to the cloud where you can list whem.', 'security-malware-firewall'),
							),
							'show_link_in_login_form' => array(
								'type' => 'field',
								'title' => __('Let them know about protection', 'security-malware-firewall'),
								'description' => __('Place a warning under login form: "Brute Force Protection by CleanTalk security. All attempts are logged".', 'security-malware-firewall'),
							),
							'set_cookies' => array(
								'type' => 'field',
								'title' => __('Set cookies', 'security-malware-firewall'),
								'description' => __('Turn this option off to forbid the plugin to create any cookies on your website front-end. This option is helpful if you use Varnish or other caching solutions. Note that disabling it will slow FireWall down a bit.', 'security-malware-firewall'),
							),
							'2fa_enable' => array(
								'type' => 'field',
								'title' => __('Two factor authentication for administrators', 'security-malware-firewall'),
								'description' => __('Use two-factor authorization via e-mail to protect Administrators.', 'security-malware-firewall'),
							),
							'complete_deactivation' => array(
								'display' => is_main_site(),
								'type' => 'field',
								'title' => __('Complete deactivation', 'security-malware-firewall'),
								'description' => __('The plugin will leave no traces in WordPress after deactivation. It could help if you have problems with the plugin.', 'security-malware-firewall'),
							),
						),
					),
				),
			),
			// Summary
			'summary' => array(
				'type' => 'tab',
				'title' => __('Summary', 'security-malware-firewall'),
				'icon' => 'icon-info',
				'class_prefix' => 'spbc',
				'ajax' => false,
				'callback' => 'spbc_tab__summary',
			),
			// Debug
			'debug' => array(
				'type' => 'tab',
				'display' => $spbc->debug,
				'title' => __('Debug', 'security-malware-firewall'),
				'class_prefix' => 'spbc',
				'ajax' => true,
				'sections' => array(
					'debug' => array(
						'type' => 'section',
						'fields' => array(
							'drop_debug' => array(
								'type' => 'field',
								'callback' => 'spbc_field_debug_drop'
							),
							'debug_data' => array(
								'type' => 'field',
								'callback' => 'spbc_field_debug'
							),
						),
					),
				),
			),
		)
	);
}

/**
 * Preprocess the lements. Registrening sections and fileds and other stuff
 * 
 * @param array $elems Array of elements
 * @param string $section_name Section name to register
 * @return array Processed elements
 */
function spbc_settings__register_sections_and_fields($elems, $section_name = '') {
	
	$elems_original = $elems;
	
	$plain_default_params = array(
		'title'   => '',
		'html'    => '',
		'display' => true,
	);
	
	$tab_default_params = array(
		'name'		  => '',
		'title'		  => '',
		'description' => '',
		'active'	  => false,
		'icon'		  => '',
		'display'	  => true,
		'preloader'	  => '<img class="spbc_spinner_big" src="' . SPBC_PATH . '/images/preloader2.gif" />',
		'ajax'		  => true,
		'js_before'	  => null,
		'js_after'	  => null,
	);
	
	$section_default_params = array(
		'title'          => '',	
		'description'    => '',
		'html_before'    => '',
		'html_after'     => '',
		'display' => true,
	);
	
	$field_default_params = array(
		'callback'    => 'spbc_settings__field__draw',
		'input_type'  => 'checkbox',
		'def_class'   => 'spbc_wrapper_field',
		'class'       => null,
		'parent'      => null,
		'childrens'   => null,
		'display'     => true, // Draw element or not
		'parent'      => null,
		'childrens'   => null,
	);
	
	
	foreach($elems as $elem_name => &$elem){
		
		// Merging with default params
		$elem = array_merge(${$elem['type'].'_default_params'}, $elem);
				
		switch ($elem['type']) {
			case 'plain':
				
				break;
			case 'tab':
				if(isset($elem['sections']))
					$elem['sections'] = spbc_settings__register_sections_and_fields($elem['sections']);
				// Creating new elements with tabs headings (before tabs)
				if($elem['display']){
					$tab_head = '<h2 class="spbc_tab_nav spbc_tab_nav-'. $elem_name .' '. (!empty($elem['active']) ? 'spbc_tab_nav--active' : '').'">'
							. '<i class="'. (isset($elem['icon']) ? $elem['icon'] : 'icon-search') .'"></i>'
							. $elem['title']
						. '</h2>';
					if(empty($elems_original['tab_headings'])){
						array_insert(
							$elems_original,
							$elem_name, 
							array('tab_headings' => array(
								'type' => 'tab_headings',
								'html' => $tab_head,
								'display' => true,
							))
						);
					}else{
						$elems_original['tab_headings']['html'] .= $tab_head;
					}
				}
				break;
			case 'section':
//				add_settings_section('spbc_section__'.$elem_name, '', 'spbc_section__'.$elem_name, 'spbc');
				if(isset($elem['fields']))
					$elem['fields'] = spbc_settings__register_sections_and_fields($elem['fields'], $elem_name);
				break;
			case 'field':
				$elem['name'] = $elem_name;
//				add_settings_field('spbc_field__'.$elem_name, '', $elem['callback'], 'spbc', 'spbc_section__'.$elem_name, $section_name);
				break;
		}
		
		$elems_original[$elem_name] = $elem;
	}
	
	return $elems_original;
	
}

/**
 * Outputs elements and tabs
 * 
 * @global type $spbc
 */
function spbc_settings__draw_elements($elems_to_draw = null, $direct_call = false) {
		
	global $spbc;
	
	if(isset($_POST['security']) && !$direct_call){
		spbc_settings__register();
		check_ajax_referer('spbc_secret_nonce', 'security');
		if(isset($_POST['tab_name']))
			$elems_to_draw = array($_POST['tab_name'] => $spbc->settings__elements[$_POST['tab_name']]);
	}
	
	foreach($elems_to_draw as $elem_name => &$elem){
		
		if(!$elem['display'])
			continue;
		
		switch ($elem['type']) {
			case 'plain':
				if(isset($elem['callback']) && function_exists($elem['callback']))
					call_user_func($elem['callback']);
				else
					echo $elem['html'];
				break;
			case 'tab_headings':
				echo '<div class="spbc_tabs_nav_wrapper">'
					. $elem['html']
				. '</div>';
				break;
			case 'tab':
				
				echo '<div class="spbc_tab spbc_tab-'. $elem_name .' '. (!empty($elem['active']) ? 'spbc_tab--active' : '') .'">';
				
					if(!$elem['ajax'] || !$direct_call){
						
						// JS before
						if(isset($elem['js_before'])){
							foreach(explode(' ', $elem['js_before']) as $script){
								echo '<script src="'. SPBC_PATH .'/js/spbc-'. $script .'?ver='. SPBC_VERSION .time().'>"></script>'; // JS before tab
							}
						}
						
						// Output
						if(!empty($elem['callback'])) call_user_func($elem['callback']);
						else                          spbc_settings__draw_elements($elem['sections'], true);
						
						// Custom elements on tab
						if(isset($elem['after'])){
							if(function_exists($elem['after'])) call_user_func($elem['after']);
							else                                echo $elem['after'];
						}
							
						// JS after
						if(isset($elem['js_after'])){
							foreach(explode(' ', $elem['js_after']) as $script){
								echo '<script src="'. SPBC_PATH .'/js/spbc-'. $script .'?ver='. SPBC_VERSION .time().'>"></script>'; // JS after tab
							}
						}
					}else 
						echo $elem['preloader'];
				echo '</div>';
				
				
				break;
			case 'section':
				echo '<div class="spbc_tab_fields_group">'
						.(!empty($elem['title']) ? '<h3 class="spbc_group_header">'. $elem['title'] .'</h3>' : '');
						spbc_settings__draw_elements($elem['fields'], true);
				echo '</div>';
				break;
			case 'field':
				call_user_func($elem['callback'], $elem);
				break;
		}
		
	}
	
	if(isset($_POST['security']) && !$direct_call)
		die();
}

function spbc_settings__field__draw($field){
	
	global $spbc;
	
	echo '<div class="'.$field['def_class'].(!empty($field['class']) ? ' '.$field['class'] : '').(isset($field['parent']) ? ' spbc_sub_setting' : '').'">';
		
		switch($field['input_type']){
			
			// Checkbox type
			case 'checkbox':
				echo '<input type="checkbox" id="spbc_setting_'.$field['name'].'" name="spbc_settings['.$field['name'].']" value="1" '
					.($spbc->settings[$field['name']] == '1' ? ' checked' : '')
					.($field['parent'] && !$spbc->settings[$field['parent']] ? ' disabled="disabled"' : '')
					.(!$field['childrens'] ? '' : ' onchange="spbcSettingsDependencies([\''.implode("','",$field['childrens']).'\'])"')
					.' />';
				echo isset($field['title'])
					? '<label for="spbc_setting_'.$field['name'].'" class="spbc_setting-field_title--'.$field['type'].'">'.$field['title'].'</label>'
					: '';
				echo isset($field['title']) 
					?'<div class="spbc_settings_description">'. $field['description'] .'</div>'
					: '';
				break;
			
			// Text type
			case 'text':
				echo '<input type="text" id="spbc_setting_'.$field['name'].'" name="spbc_settings['.$field['name'].']" '
					.'value="'.($spbc->settings[$field['name']] ? $spbc->settings[$field['name']] : '').'" '
					.($field['parent'] && !$spbc->settings[$field['parent']] ? ' disabled="disabled"' : '')
					.(!$field['childrens'] ? '' : ' onchange="spbcSettingsDependencies([\''.implode("','",$field['childrens']).'\'])"')
					.' />'
				.'&nbsp;<label for="spbc_setting_'.$field['name'].'" class="spbc_setting-field_title--'.$field['type'].'">'
					.$field['title']
				.'</label>';
				if(isset($field['description']))
					echo '<div class="spbc_settings_description">'.$field['description'].'</div>';
				break;
		}
		
	echo '</div>';
}

/**
 * Admin callback function - Displays plugin options page
 */
function spbc_settings_page() {
	
	global $spbc;
	
	// If it's network admin dashboard
	if(is_network_admin()){
		$link = get_site_option('siteurl').'wp-admin/options-general.php?page=spbc';
		printf("<h2>" . __("Please, enter the %splugin settings%s in main site dashboard.", 'security-malware-firewall') . "</h2>", "<a href='$link'>", "</a>");
		return;
	}
	
	// Waringns counter on Summary tab
	$warnings = '';
	$warnings .= !empty($spcb->data['warnings']['black'])  ? sprintf('<span class="spbc_warning_counter spbc_warning_counter--black">%s</span>',  $spcb->data['warnings']['black'])  : '';
	$warnings .= !empty($spcb->data['warnings']['red'])    ? sprintf('<span class="spbc_warning_counter spbc_warning_counter--red">%s</span>',    $spcb->data['warnings']['red'])    : '';
	$warnings .= !empty($spcb->data['warnings']['orange']) ? sprintf('<span class="spbc_warning_counter spbc_warning_counter--orange">%s</span>', $spcb->data['warnings']['orange']) : '';
	$warnings .= !empty($spcb->data['warnings']['green'])  ? sprintf('<span class="spbc_warning_counter spbc_warning_counter--green">%s</span>',  $spcb->data['warnings']['green'])  : '';
	
	$warnings .= sprintf('<span class="spbc_warning_counter spbc_warning_counter--black">%s</span>',  1);
	$warnings .= sprintf('<span class="spbc_warning_counter spbc_warning_counter--red">%s</span>',    2);
	$warnings .= sprintf('<span class="spbc_warning_counter spbc_warning_counter--orange">%s</span>', 3);
	$warnings .= sprintf('<span class="spbc_warning_counter spbc_warning_counter--green">%s</span>',  4);
	
	
	// allow_url_fopen error
	$allow_url_fopen = strtolower(ini_get('allow_url_fopen'));
	if(($allow_url_fopen !== 'on' && $allow_url_fopen !== '1') && is_admin()){
		$spbc->error_add('allow_url_fopen', '');
	}else{
		$spbc->error_delete('allow_url_fopen');
	}
	
	// Low memory limit error
	$m_limit = ini_get('memory_limit');
	if(is_string($m_limit)){
		$prefix = strtolower(substr($m_limit, -1, 1));
		$numder = substr($m_limit, 0, -1);
		switch($prefix){
			case 'k': $m_limit = $numder * 1000; break;
			case 'm': $m_limit = $numder * 1000000; break;
			case 'g': $m_limit = $numder * 1000000000; break;
		}
	}
	if($m_limit - memory_get_usage(true) < 25 * 1024 * 1024 ){
		$spbc->error_add('memory_limit_low', '');
	}else{
		$spbc->error_delete('memory_limit_low');
	}
	
	// Outputs errors if exists
	spbc_settings__error__output();
	
	echo ''
		. '<div id="gdpr_dialog" class="spbc_hide" style="padding: 0 15px;">'
			. spbc_show_GDPR_text()
		. '</div>'
		
		. '<div class="wrap">'
			. '<form id="spbc_settings_form" method="post" action="options.php">'	
				. '<h2 style="display: inline-block;">'. SPBC_NAME. '</h2>'
				. '<div style="float: right; margin: 10px 0 0 0; font-size: 13px;">'
					. '<b style="display: inline-block;">'
						.sprintf(
							__('Do you like CleanTalk? %sPost your feedback here%s.', 'cleantalk'),
							'<a href="https://wordpress.org/support/plugin/security-malware-firewall/reviews/#new-post" target="_blank">',
							'</a>'
						)
					. '</b>'
					. '<br />'
					.spbc_badge__get_premium()
				. '</div>'
				. '</br>';
				
				settings_fields(SPBC_SETTINGS);
				spbc_settings__draw_elements($spbc->settings__elements, 'direct_call');
						
			echo '</form>'
			. '<form id="drop_debug" method="POST"></form>'
		. '</div>';
}

/**
* Message output error block.
* 
* @global SpbcState $spbc
* @return void
*/
function spbc_settings__error__output(){	
	
	global $spbc;
	
	if(!empty($spbc->data['errors'])){
		
		$errors = $spbc->data['errors'];
		
		$error_texts = array(
			// Scanner
			'perform_scan_wrapper' => __('Error occured while scanning. Error: ', 'security-malware-firewall'),
			'get_hashs' => __('Error occured while getting remote hashs. Error: ', 'security-malware-firewall'),
			'scan' => __('Error occured while scanning. Error: ', 'security-malware-firewall'),
			'count_unchecked' => __('Error occured while counting uncheccked files. Error: ', 'security-malware-firewall'),
			'scan_modified' => __('Error occured while scanning modified files. Error: ', 'security-malware-firewall'),
			'scanner_result_send' => __('Error occurred while sending scan logs. Error: ', 'security-malware-firewall'),
			'allow_url_fopen' => __('PHP setting "allow_url_fopen" is disabled. This could effect malware scan quality.', 'security-malware-firewall'),
			'memory_limit_low' => __('You have less than 25 Mib free PHP memory. Error could occurs while scanning.', 'security-malware-firewall'),
			// Misc
			'apikey' => __('Error occured while API key validating. Error: ', 'security-malware-firewall'),
			'get_key' => __('Error occured while automatically gettings access key. Error: ', 'security-malware-firewall'),
			'send_logs' => __('Error occured while sending security logs. Error: ', 'security-malware-firewall'),
			'send_firewall_logs' => __('Error occured while sending firewall logs. Error: ', 'security-malware-firewall'),
			'firewall_update' => __('Error occured while updating firewall. Error: '            , 'security-malware-firewall'),
			'notice_paid_till' => __('Error occured while checking account status. Error: ', 'security-malware-firewall'),
			'access_key_notices' => __('Error occured while checking account status. Error: ', 'security-malware-firewall'),
			// Cron
			'send_php_logs' => __('Error occured while sending PHP error log. Error: ', 'security-malware-firewall'),
			// Unknown
			'unknown' => __('Unknown error. Error: ', 'security-malware-firewall'),
		);
		
		$errors_out = array();
		
		foreach($errors as $type => $error){
			
			if(!empty($error)){
				
				if(is_array(current($error))){
					
					foreach($error as $sub_type => $sub_error){
						$errors_out[$sub_type] = '';
						if(isset($sub_error['error_time']))
							$errors_out[$sub_type] .= date('Y-m-d H:i:s', $sub_error['error_time']) . ': ';
						$errors_out[$sub_type] .= ucfirst($type).': ';
						$errors_out[$sub_type] .= (isset($error_texts[$sub_type]) ? $error_texts[$sub_type] : $error_texts['unknown']) . $sub_error['error_string'];
					}
					continue;
				}
				
				$errors_out[$type] = '';
				if(isset($error['error_time'])) 
					$errors_out[$type] .= date('Y-m-d H:i:s', $error['error_time']) . ': ';
				$errors_out[$type] .= (isset($error_texts[$type]) ? $error_texts[$type] : $error_texts['unknown']) . (isset($error['error_string']) ? $error['error_string'] : '');
				
			}
		}
		
		if(!empty($errors_out)){
			echo '<div id="spbcTopWarning" class="error" style="position: relative;">'
				.'<h3 style="display: inline-block;">'.__('Errors:', 'security-malware-firewall').'</h3>';
				foreach($errors_out as $value){
					echo '<h4>'.$value.'</h4>';
				}
				echo '<h4 style="text-align: right;">'.sprintf(__('You can get support any time here: %s.', 'security-malware-firewall'), '<a target="blank" href="https://wordpress.org/support/plugin/security-malware-firewall">https://wordpress.org/support/plugin/security-malware-firewall</a>').'</h4>';
			echo '</div>';
		}
	}
}

/**
 * Admin callback function - Displays field of security status
 */
function spbc_field_security_status(){
	
	global $spbc;
	
	// Setting img's paths
	$path_to_img = SPBC_PATH . '/images/';
	$img = $path_to_img.'yes.png';
	$img_no = $path_to_img.'no.png';
	$img_no_gray = $path_to_img.'no_gray.png';
	
	// Setting statuses
	$scanner_status = ($spbc->key_is_ok && $spbc->scaner_status && !empty($spbc->data['scanner']['last_scan']) && $spbc->data['scanner']['last_scan'] + (86400*7) > current_time('timestamp')) || $spbc->scaner_warning ? true : false;
	$ssl_status = is_ssl();
	$ssl_text   = sprintf('%s' . __('SSL Installed', 'security-malware-firewall') . '%s', 
		$ssl_status || !$spbc->key_is_ok ? '' : '<a href="https://cleantalk.org/my/?cp_mode=ssl'.($spbc->user_token ? '&user_token='.$spbc->user_token : '').'" target="_blank">',
		$ssl_status || !$spbc->key_is_ok ? '' : '</a>'
	);
	
	// Output statuses
	echo '<h2 style="display: inline-block;">'.__('Status:', 'security-malware-firewall').'</h2>';
	
	echo '<div style="display: inline-block;">';
	
		echo '<img class="spbc_status_icon" src="'.($spbc->key_is_ok ? $img : $img_no).'"/>'
			.'<a href="#" onclick="spbc_switchTab(document.getElementsByClassName(\'spbc_tab_nav-security_log\')[0])">'
				.__('Brute Force Protection', 'security-malware-firewall')
			.'</a>';
			
		echo '<img class="spbc_status_icon" src="'.($spbc->key_is_ok ? $img : $img_no).'"/>'
			.'<a href="#" onclick="spbc_switchTab(document.getElementsByClassName(\'spbc_tab_nav-traffic_control\')[0])">'
				.__('FireWall', 'security-malware-firewall')
			.'</a>';
			
		if($spbc->scaner_enabled){
			echo '<img class="spbc_status_icon" id="spbc_scanner_status_icon" src="'.($scanner_status ? $img : $img_no).'"/>'
			.'<a href="#" onclick="spbc_switchTab(document.getElementsByClassName(\'spbc_tab_nav-scanner\')[0])">'
				.__('Malware Scanner', 'security-malware-firewall')
			.'</a>';
		}
		
		echo '<img class="spbc_status_icon" src="'.($spbc->key_is_ok ? $img : $img_no).'"/>'.__('Security Report', 'security-malware-firewall');
		
		echo '<img class="spbc_status_icon" src="'.($spbc->key_is_ok ? $img : $img_no).'"/>'
			.'<a href="#" onclick="spbc_switchTab(document.getElementsByClassName(\'spbc_tab_nav-security_log\')[0])">'	
				.__('Security Audit Log', 'security-malware-firewall')
			.'</a>';
			
		echo '<img class="spbc_status_icon" src="'.($ssl_status      ? $img : $img_no).'"/>'.$ssl_text;
		
		// Autoupdate status
		if($spbc->notice_auto_update == 1){
			echo '<img class="spbc_status_icon" src="'.($spbc->auto_update == 1 ? $img : ($spbc->auto_update == -1 ? $img_no : $img_no_gray)).'"/>'
				.'<a href="http://cleantalk.org/help/cleantalk-auto-update" target="_blank">'
					.__('Auto update', 'security-malware-firewall')
				.'</a>';
		}
		
	echo '</div>';
	echo '<br>';
}

function spbc_tab__summary(){
	echo '<div class="spbc_tab_fields_group">'
		. '<h3 class="spbc_group_header">'. __('Statistics', 'security-malware-firewall') .'</h3>';
		spbc_field_statistics();
	echo '</div>';
	echo '<br>';
	echo '<span id="spbc_gdpr_open_modal" style="text-decoration: underline">'.__('GDPR compliance', 'security-malware-firewall').'</span>';
	echo '<br>';
	echo __('Tech support:', 'security-malware-firewall') . ' <a target="_blank" href="https://wordpress.org/support/plugin/security-malware-firewall">Wordpress.org</a>';
	echo '<br>';
	printf(__('The plugin home page', 'security-malware-firewall') .' <a href="https://wordpress.org/plugins/security-malware-firewall/" target="_blank">%s</a>.', SPBC_NAME);
	echo '<br>';
	echo __('CleanTalk is registered Trademark. All rights reserved.', 'security-malware-firewall');
	echo '<br>';
}

/**
 * Admin callback function - Displays current statistics
 */
function spbc_field_statistics(){
	
	global $spbc;
	
	echo "<div class='spbc_wrapper_field'>";
	
		// Info block
		echo (isset($spbc->data['logs_last_sent'], $spbc->data['last_sent_events_count'])
			? sprintf(__('%d events have been sent to CleanTalk Cloud on %s.', 'security-malware-firewall'), $spbc->data['last_sent_events_count'], date("M d Y H:i:s", $spbc->data['logs_last_sent']))
			: __('Unknow last logs sending time.', 'security-malware-firewall'));
		echo '<br />';
		echo (isset($spbc->data['last_firewall_send'], $spbc->data['last_firewall_send_count'])
			? sprintf(__('Information about %d blocked entries have been sent to CleanTalk Cloud on %s.', 'security-malware-firewall'), $spbc->data['last_firewall_send_count'], date("M d Y H:i:s", $spbc->data['last_firewall_send']))
			: __('Unknow last filrewall logs sending time.', 'security-malware-firewall'));
		echo '<br />';
		echo (isset($spbc->data['last_firewall_updated'], $spbc->data['firewall_entries'])
			? sprintf(__('Security FireWall database has %d IPs. Last updated at %s.', 'security-malware-firewall'), $spbc->data['firewall_entries'], date('M d Y H:i:s', $spbc->data['last_firewall_updated']))
			: __('Unknow last Security FireWall updating time.', 'security-malware-firewall'));
		if($spbc->scaner_enabled){
			echo '<br />';
			echo (isset($spbc->data['scanner']['last_scan'])
				? sprintf(__('Website last scan was performed on %s', 'security-malware-firewall'), date('M d Y H:i:s', $spbc->data['scanner']['last_scan']))
				: __('Website hasn\'t been scanned yet.', 'security-malware-firewall'));
		
			if(isset($spbc->data['scanner']['last_sent'])){
				echo '<br />';
				printf(__('Scan results were sent to the cloud at %s', 'security-malware-firewall'), date('M d Y H:i:s', $spbc->data['scanner']['last_sent']));
			}
		}
	
	echo '</div>';
}

function spbc_field_banners(){
	global $spbc_tpl;
	// Rate banner
	// echo sprintf($spbc_tpl['spbc_rate_plugin_tpl'],
		// SPBC_NAME  
	// );
	// Translate banner
	if(substr(get_locale(), 0, 2) != 'en'){
		echo sprintf($spbc_tpl['spbc_translate_banner_tpl'],
			substr(get_locale(), 0, 2)
		);
	}
}

/**
 * Admin callback function - Displays field of Api Key
 */
function spbc_field_key( $values = null ) {

	global $spbc;
	
	echo "<div class='spbc_wrapper_field'>";
	
		if($spbc->allow_custom_key || is_main_site()){
			
			// Key is OK
			if($spbc->key_is_ok){
				echo '<input id="spbc_key" name="spbc_settings[spbc_key]" size="20" type="text" value="'.str_repeat('*', strlen($spbc->settings['spbc_key'])).'" key="'.$spbc->settings['spbc_key'].'" style="font-size: 14pt;" placeholder="' . __('Enter the key', 'security-malware-firewall') . '" />';
				
				if(!empty($spbc->data['account_name_ob'])){
					echo '<div class="spbc_hide">'
						.sprintf(
							__('Account at cleantalk.org is %s.', 'cleantalk'),
							'<b>'.$spbc->data['account_name_ob'].'</b>'
						)
					.'</div>';
				}
				echo '<a id="showHideLink" class="spbc-links" style="color:#666;" href="#">'.__('Show access key', 'security-malware-firewall').'</a>';
			
			// Key is not OK
			}else{
				
				echo '<input id="spbc_key" name="spbc_settings[spbc_key]" size="20" type="text" value="'.$spbc->settings['spbc_key'].'" style=\'font-size: 14pt;\' placeholder="' . __('Enter the key', 'security-malware-firewall') . '" />';
				echo '<br/><br/>';
				echo '<a target="_blank" href="https://cleantalk.org/register?platform=wordpress&email='.urlencode(get_option('admin_email')).'&website='.urlencode(parse_url(get_option('siteurl'), PHP_URL_HOST)).'&product_name=security" style="display: inline-block;">
						<input type="button" class="spbc_auto_link" value="'.__('Get access key manually', 'security-malware-firewall').'" />
					</a>';
				echo '&nbsp;'.__('or', 'security-malware-firewall').'&nbsp;';
				echo '<input id="get_key_auto" name="spbc_get_apikey_auto" type="submit" class="spbc_manual_link" value="' . __('Get access key automatically', 'security-malware-firewall') . '" />';
				echo '<br/><br/>';
				echo '<div style="font-size: 10pt; color: #666 !important">'
					.sprintf(
						__('Admin e-mail (%s) will be used for registration', 'security-malware-firewall'),
						get_option('admin_email')
					)
				.'</div>';
				echo '<div>';
					echo '<input checked type="checkbox" id="license_agreed" onclick="spbcSettingsDependencies(\'get_key_auto\');"/>';
					echo '<label for="spbc_license_agreed">';
						printf(
							__('I agree with %sPrivacy Policy%s of %sLicense Agreement%s', 'security-malware-firewall'),
							'<a href="https://cleantalk.org/publicoffer#privacy" target="_blank" style="color:#66b;">', '</a>',
							'<a href="https://cleantalk.org/publicoffer"         target="_blank" style="color:#66b;">', '</a>'
						);
					echo "</label>";
				echo '</div>';
				
				echo '<input type="hidden" id="spbc_admin_timezone" name="ct_admin_timezone" value="null" />';
			}
			
		}else{
			_e('<h3>Key is provided by Super Admin.<h3>', 'security-malware-firewall');
		}
		
	echo '</div>';
	
}

function spbc_field_security_logs__prepare_data(&$table){
	
	if($table->items_count){
		
		foreach($table->rows as $row){
			$ips_c[] = $row->auth_ip;
		} unset($row);
		$ips_c = spbc_get_countries_by_ips(implode(',', $ips_c));
		
		$time_offset = current_time('timestamp') - time();
		
		foreach($table->rows as $row){
			
			$user = get_user_by('login', $row->user_login);
			$user_part = sprintf("<a href=\"%s\">%s</a>",
				$user ? (admin_url() . '/user-edit.php?user_id=' .  $user->data->ID) : '#',
				$row->user_login
			);
			
			$page      = $row->page      === null ? '-'           : "<a href='".$row->page."' target='_blank'>".$row->page."</a>";
			
			
			switch($row->event){
				case 'view':
					$event = 'Viewing admin page ('.($row->page_time === null ? 'Calculating' : strval($row->page_time).' secoonds').')';
					break;
				case 'auth_failed':
					$event = 'Failed authentification';
					break;
				case 'auth_failed_2fa':
					$event = 'Failed two factor authentification';
					break;
				case 'login_2fa':
					$event = 'Two factor authentification';
					break;
				default:
					$event = $row->event;
					break;
			}
			
			$country_part = spbc_report_country_part($ips_c, $row->auth_ip);
			$ip_part = sprintf("<a href=\"https://cleantalk.org/blacklists/%s\" target=\"_blank\">%s</a>,&nbsp;%s",
				$row->auth_ip, 
				SpbcHelper::ip__v6_reduce($row->auth_ip), 
				$country_part
			);
			
			$table->items[] = array(
				'user_login' => $user_part,
				'datetime'   => date("M d Y, H:i:s", strtotime($row->datetime) + $time_offset),
				'event'      => $event,
				'page'       => $page,
				'auth_ip'    => $ip_part,
			);
		}
	}
}

/**
 * Admin callback function - Displays description of 'main' plugin parameters section
 */
 
function spbc_field_security_logs($value = array('id' => 'spbc_option_security_logs', 'class' => 'spbc-settings-section')){
	
	global $spbc, $wpdb;
	
	echo '<div class="spbc_wrapper_field">';
	
		if(!$spbc->key_is_ok){
			$button = '<input type="button" class="button button-primary" value="'.__('To setting', 'security-malware-firewall').'"  />';
			$link = sprintf('<a href="#" onclick="spbc_switchTab(document.getElementsByClassName(\'spbc_tab_nav-settings_general\')[0], \'spbc_key\', 3);">%s</a>', $button);
			echo '<div style="margin: 10px auto; text-align: center;"><h3 style="margin: 5px; display: inline-block;">'.__('Please, enter API key.', 'security-malware-firewall').'</h3>'.$link.'</div>';
			return;
		}

		// HEADER
		$message_about_log = sprintf(__('This list contains details of Brute force attacks and security actions for the past 24 hours and shows only last %d records. To see the full report please use %sSecurity control panel%s.', 'security-malware-firewall'),
			SPBC_LAST_ACTIONS_TO_VIEW,
			'<a target="_blank" href="https://cleantalk.org/my/logs?user_token='.$spbc->user_token.'">',
			'</a>'
		);
		echo "<p class='spbc_hint spbc_hint-security_logs -display--inline-block'>$message_about_log</p>";		

		// OUTPUT
		$table = new SpbcListTable(
			array(
				'id' => 'spbc_tbl__secuirty_logs',
				'sql' => array(
					'add_col' => array('id', 'page_time'),
					'table'   => SPBC_TBL_SECURITY_LOG,
					'where'   => (SPBC_WPMS ? ' WHERE blog_id = '.get_current_blog_id() : ''),
					'offset'  => 0,
					'limit'   => SPBC_LAST_ACTIONS_TO_VIEW,
					'get_array' => false,
				),
				'order_by'  => array('datetime' => 'desc'),
				'func_data_prepare' => 'spbc_field_security_logs__prepare_data',
				'if_empty_items' => '<p class="spbc_hint">'.__("0 brute-force attacks have been made.", 'security-malware-firewall').'</p>',
				'columns' => array(
					'user_login' => array('heading' => 'User', 'primary' => true,),
					'auth_ip'    => array('heading' => 'IP',),
					'datetime'   => array('heading' => 'Date',),
					'event'      => array('heading' => 'Action',),
					'page'       => array('heading' => 'Page',),
				),
				'sortable' => array('user_login', 'datetime'),
			)
		);

		$table->get_data();

		// Send logs button
		if($table->items_total)
			echo '<p class="spbc_hint spbc_hint-send_security_log spbc_hint--link spbc_hint--top_right">'
				.__('Send logs', 'security-malware-firewall')
			.'</p>';

		$table->display();

		// SHOW MORE	
		if($table->items_total > SPBC_LAST_ACTIONS_TO_VIEW){
			echo '<div class="spbc__wrapper--center spbc__wrapper--show_more">';
				if(!empty($spbc->user_token)){
					echo '<div class="spbc__show_more_logs">'
						."<h3 class='-display--inline-block'>"
							.__('Proceed to:', 'security-malware-firewall')."&nbsp;"
						."</h3>"
						."<a target='_blank' href='https://cleantalk.org/my/logs?service=".$spbc->service_id."&user_token=".$spbc->user_token."' class='spbc_manual_link -display--inline-block'>"
							.__('Security Control Panel', 'security-malware-firewall')
						."</a>"
						."<h3 class='-display--inline-block'>&nbsp;"
							.__('to see more.', 'security-malware-firewall')
						."</h3>"
					.'</div>';
				}
				echo "<div id='spbc_show_more_button' class='spbc_manual_link'>"
					.__('Show more', 'security-malware-firewall')
				."</div>"
				.'<img class="spbc_preloader" src="'.SPBC_PATH.'/images/preloader.gif" />'
			."</div>";
		}
	
	echo '</div>';
	
}

function spbc_field_traffic_control_logs__prepare_data(&$table){
	
	global $spbc;
		
	if($table->items_count){
		
		foreach($table->rows as $row)
			$ip_countries[] = $row->ip_entry;
		$ip_countries = spbc_get_countries_by_ips(implode(',', $ip_countries));
		
		$time_offset = current_time('timestamp') - time();
		
		foreach($table->rows as $row){
			
			$ip = "<a href='https://cleantalk.org/blacklists/{$row->ip_entry}' target='_blank'>".SpbcHelper::ip__v6_reduce($row->ip_entry).'</a>'
				.'&nbsp;<sup>'
					."<a href='https://cleantalk.org/my/show_private?service_id={$spbc->service_id}&add_record={$row->ip_entry}&service_type=securityfirewall' target='_blank' class='spbc_gray'>"
						.__('Manage', 'security-malware-firewall').
					'</a>'
				 .'</sup>';
			
			$requests = '<b>'.$row->requests.'</b>';
			
			$page_url =	strlen($row->page_url) >= 60
					 ? '<div class="spbcShortText">'.substr($row->page_url, 0, 60).'...</div>'
					  .'<div class="spbcFullText spbc_hide">'.$row->page_url.'</div>'
					 : $row->page_url;
					 
			$user_agent = strlen($row->http_user_agent) >= 60
					? '<div class="spbcShortText">'.substr($row->http_user_agent, 0, 60).'...</div>'
					 .'<div class="spbcFullText spbc_hide">'.$row->http_user_agent.'</div>'
					: $row->http_user_agent;
			
			switch($row->status){
				case 'PASS':                    $status = '<span class="spbcGreen">' . __('Passed', 'security-malware-firewall').'</span>'; 
					break;
				case 'PASS_BY_TRUSTED_NETWORK': $status = '<span class="spbcGreen">' . __('Passed. Trusted network. Click on IP for details.', 'security-malware-firewall').'</span>';
					break;
				case 'PASS_BY_WHITELIST':       $status = '<span class="spbcGreen">' . __('Passed. Whitelisted.', 'security-malware-firewall').'</span>';
					break;
				case 'DENY':                    $status = '<span class="spbcRed">'   . __('Blocked. Blacklisted.', 'security-malware-firewall').'</span>';
					break;
				case 'DENY_BY_NETWORK':	        $status = '<span class="spbcRed">'   . __('Blocked. Hazardous network.', 'security-malware-firewall').'</span>';
					break; 
				case 'DENY_BY_DOS':             $status = '<span class="spbcRed">'   . __('Blocked by DoS prevention system', 'security-malware-firewall').'</span>'; 
					break;
					
				// WAF
				case 'DENY_BY_WAF_XSS':
					$status = '<span class="spbcRed">' . __('Blocked by Web Application Firewall: XSS attack detected.', 'security-malware-firewall')    . '</span>';
					break;
				case 'DENY_BY_WAF_SQL':
					$status = '<span class="spbcRed">' . __('Blocked by Web Application Firewall: SQL-injection detected.', 'security-malware-firewall') . '</span>';
					break;
				case 'DENY_BY_WAF_FILE':
					$status = '<span class="spbcRed">'
						.__('Blocked by Web Application Firewall: ', 'security-malware-firewall')
						.'<span class="spbc_waf_reason_title">'
							.__('Malicious files upload.', 'security-malware-firewall')
						.'</span>'
						.' <span class="spbc_waf_reason">'
							.__('Reason: ', 'security-malware-firewall')
							// .json_decode($row->pattern, true)
							.$row->pattern
						.'</span>'
						.''
					.'</span>';
					break;
				default: $status = __('Unknown', 'security-malware-firewall'); break;
			}
			
			$table->items[] = array(
				'ip_entry'        => $ip,
				'country'         => spbc_report_country_part($ip_countries, $row->ip_entry),
				'entry_timestamp' => date('M d Y, H:i:s', $row->entry_timestamp + $time_offset),
				'requests'        => $requests,
				'status'          => $status,
				'page_url'        => $page_url,
				'http_user_agent' => $user_agent,
			);
		}		
	}
}

function spbc_field_traffic_control_log( $value = array() ){
	
	global $spbc, $wpdb, $spbc_tpl;
	
	echo '<div class="spbc_wrapper_field">';
	
	if(!$spbc->key_is_ok){
		$button = '<input type="button" class="button button-primary" value="'.__('To setting', 'security-malware-firewall').'"  />';
		$link = sprintf('<a href="#" onclick="spbc_switchTab(document.getElementsByClassName(\'spbc_tab_nav-settings_general\')[0], \'spbc_key\', 3);">%s</a>', $button);
		echo '<div style="margin: 10px auto; text-align: center;"><h3 style="margin: 5px; display: inline-block;">'.__('Please, enter API key.', 'security-malware-firewall').'</h3>'.$link.'</div>';
	}elseif(!$spbc->tc_status){
		$button = '<input type="button" class="button button-primary" value="'.__('RENEW', 'security-malware-firewall').'"  />';
		$link = sprintf('<a target="_blank" href="https://cleantalk.org/my/bill/security?cp_mode=security&utm_source=wp-backend&utm_medium=cpc&utm_campaign=WP%%20backend%%20trial_security&user_token=%s">%s</a>', $spbc->user_token, $button);
		echo '<div style="margin-top: 10px;">'
			.'<h3 style="margin: 5px; display: inline-block;">'.__('Please renew your security license.', 'security-malware-firewall').'</h3>'.$link.
		'</div>';
	}else{		
		
		$table = new SpbcListTable(
			array(
				'id' => 'spbc_tbl__traffic_control_logs',
				'sql' => array(
					'except_cols' => array('country', 'entries'),
					'add_col'     => array('entry_id','pattern'),
					'table'       => SPBC_TBL_FIREWALL_LOG,
					'offset'      => 0,
					'limit'       => SPBC_LAST_ACTIONS_TO_VIEW,
					'get_array'  => false,
				),
				'order_by'  => array('entry_timestamp' => 'desc'),
				'func_data_prepare' => 'spbc_field_traffic_control_logs__prepare_data',
				'if_empty_items' => '<p class=spbc_hint>'.__("Local log is empty.", 'security-malware-firewall').'</p>',
				'columns' => array(
					'ip_entry' => array(
						'heading' => 'IP',
						'primary' => true,
					),
					'country'         => array('heading' => 'Country',),
					'entry_timestamp' => array('heading' => 'Last Request',),
					'status'          => array('heading' => 'Status',),
					'requests'        => array('heading' => 'Requests', 'class' => ' tbl-width--100px'),
					'page_url'        => array('heading' => 'Page',),
					'http_user_agent' => array('heading' => 'User Agent',),
				),
				'sortable' => array('status', 'entry_timestamp'),
			)
		);
		
		$table->get_data();
		
		if($table->items_total){
			echo '<p class="spbc_hint spbc_hint--left -display--inline-block">';
				printf(__('This list contains details of access attempts for the past hour and shows only last %d records.', 'security-malware-firewall'),
						SPBC_LAST_ACTIONS_TO_VIEW
					);
					echo "&nbsp;";
				printf(__('The list updates itself every %d seconds automatically.', 'security-malware-firewall'), 60);
			echo '</p>';
			echo "<p class='spbc_hint spbc_hint-send_traffic_control spbc_hint--link spbc_hint--top_right'>Send logs</p>"; // Send logs button
			
		}
		
		echo '<p class="spbc_hint spbc_hint--left -display--inline-block">&nbsp;'
			.sprintf(
				__('Traffic Control will block visitors if they send more than %s requests per hour.', 'security-malware-firewall'),
				'<b>'.(isset($spbc->settings['traffic_control_autoblock_amount']) ? $spbc->settings['traffic_control_autoblock_amount'] : 1000).'</b>'
			)
			.' '
			.sprintf(
				__('You can adjust it %shere%s.', 'security-malware-firewall'),
				'<a href="#" onclick="spbc_switchTab(document.getElementsByClassName(\'spbc_tab_nav-settings_general\')[0], \'spbc_setting_traffic_control_autoblock_amount\', 3);">',
				'</a>'
			)
		.'</p>';
		echo '<br>';
		echo '<p class="spbc_hint spbc_hint--left -display--inline-block">&nbsp;'
			.sprintf(
				__('Traffic Control is %s.', 'security-malware-firewall'),
				'<b>'.(!empty($spbc->settings['traffic_control_enabled'])
					? __('active', 'security-malware-firewall')
					: __('inactive', 'security-malware-firewall')
					)
				.'</b>'
			)
			.(!empty($spbc->settings['traffic_control_enabled'])
				? ''
				: ' ' . sprintf(
					__('You can activate it %shere%s.', 'security-malware-firewall'),
					'<a href="#" onclick="spbc_switchTab(document.getElementsByClassName(\'spbc_tab_nav-settings_general\')[0], \'spbc_setting_traffic_control_enabled\', 3);">',
					'</a>'
				)
			)
		.'</p>';
		echo '<br>';
		echo '<p class="spbc_hint spbc_hint--left -display--inline-block">&nbsp;'
			.sprintf(
				__('Web Application Firewall (WAF) is %s.', 'security-malware-firewall'),
				'<b>'.(!empty($spbc->settings['waf_enabled'])
					? __('active', 'security-malware-firewall')
					: __('inactive', 'security-malware-firewall')
					)
				.'</b>'
			)
			.(!empty($spbc->settings['waf_enabled'])
				? ''
				: ' ' . sprintf(
					__('You can activate it %shere%s.', 'security-malware-firewall'),
					'<a href="#" onclick="spbc_switchTab(document.getElementsByClassName(\'spbc_tab_nav-settings_general\')[0], \'spbc_setting_waf_enabled\', 3);">',
					'</a>'
				)
			)
		.'</p>';
		
		$table->display();
		
		if($table->items_total > SPBC_LAST_ACTIONS_TO_VIEW){
			echo "<div class='spbc__wrapper--center spbc__wrapper--show_more'>";
				if($spbc->user_token){
					echo '<div class="spbc__show_more_logs">'
						.'<h3 class="-display--inline-block">'
							.__('Proceed to:', 'security-malware-firewall').'&nbsp;'
						.'</h3>'
						.'<a target="_blank" href="https://cleantalk.org/my/logs_firewall?service='.$spbc->service_id.'&user_token='.$spbc->user_token.'" class="spbc_manual_link -display--inline-block">'
							.__('Security Control Panel', 'security-malware-firewall')
						.'</a>'
						.'<h3 class="-display--inline-block">&nbsp;'
							.__('to see more.', 'security-malware-firewall')
						.'</h3>'
					.'</div>';
				}
				echo "<div id='spbc_show_more_fw_logs_button' class='spbc_manual_link'>"
					.__('Show more', 'security-malware-firewall')
				."</div>"
				.'<img class="spbc_preloader" src="'.SPBC_PATH.'/images/preloader.gif" />'
			."</div>";
		}
	}
	echo '</div>';	
}

function spbc_field_scanner__prepare_data__files(&$table){
	
	global $wpdb;
	
	if($table->items_count){
		$root_path = spbc_get_root_path();
		
		$signatures = $wpdb->get_results('SELECT * FROM '. SPBC_TBL_SCAN_SIGNATURES, OBJECT_K);
		
		foreach($table->rows as $key => $row){
			
			// Filtering row actions
			if($row->last_sent > $row->mtime || $row->size == 0 || $row->size > 1048570) unset($row->actions['send']);
			if(!$row->real_full_hash || !$row->difference) unset($row->actions['compare']);
			if(!$row->real_full_hash) unset($row->actions['replace']);
			if(!$row->severity) unset($row->actions['view_bad']);
			if($row->status === 'quarantined') unset($row->actions['quarantine']);
			
			$table->items[] = array(
				'cb'       => $row->fast_hash,
				'uid'      => $row->fast_hash,
				'size'     => substr(number_format($row->size, 2, ',', ' '), 0, -3),
				'perms'    => $row->perms,
				'mtime'    => date('M d Y H:i:s', $row->mtime),
				'path'     => strlen($root_path.$row->path) >= 40
					? '<div class="spbcShortText">...'.$row->path.'</div><div class="spbcFullText spbc_hide">'.$root_path.$row->path.'</div>'
					: $root_path.$row->path,
				'actions' => $row->actions,
			);
						
			if(isset($row->weak_spots)){
				$weak_spots = json_decode($row->weak_spots, true);
				if($weak_spots){
					if(!empty($weak_spots['SIGNATURES'])){
						foreach ($weak_spots['SIGNATURES'] as $string => $weak_spot_in_string) {
							foreach ($weak_spot_in_string as $weak_spot) {
								$ws_string = '<span class="spbcRed">'. $signatures[$weak_spot]->attack_type .': </span>'
									.(strlen($signatures[$weak_spot]->name) > 30 
										? substr($signatures[$weak_spot]->name, 0, 30).'...'
										: $signatures[$weak_spot]->name);
							}
						}
					}elseif(!empty($weak_spots['CRITICAL'])){
						foreach ($weak_spots['CRITICAL'] as $string => $weak_spot_in_string) {
							foreach ($weak_spot_in_string as $weak_spot) {
								$ws_string = '<span class="spbcRed">Heuristic: </span>'
									.(strlen($weak_spot) > 30
										? substr($weak_spot, 0, 30).'...' 
										: $weak_spot);
							}
						}
					}else{
						$ws_string = '';
					}
				}else
					$ws_string = '';
				
				$table->items[$key]['weak_spots'] = $ws_string;
			}
		}
	}	
}

function spbc_field_scanner__prepare_data__files_qurantine(&$table){
	
	if($table->items_count){
		$root_path = spbc_get_root_path();
		foreach($table->rows as $key => $row){
			
			$table->items[] = array(
				'cb'       => $row->fast_hash,
				'uid'      => $row->fast_hash,
				'actions'  => $row->actions,
				'path'     => strlen($root_path.$row->path) >= 40
					? '<div class="spbcShortText">...'.$row->path.'</div><div class="spbcFullText spbc_hide">'.$root_path.$row->path.'</div>'
					: $root_path.$row->path,
				'q_status' => $row->q_status,
				'severity' => $row->severity,
				'q_time'   => date('M d Y H:i:s', $row->q_time),
				'size'     => substr(number_format($row->size, 2, ',', ' '), 0, -3),
			);
		}
	}	
}

function spbc_field_scanner__prepare_data__domains(&$table){
	if($table->items_count){
		$num = $table->sql['offset']+1;
		foreach($table->rows as $row){
			$table->items[] = array(
				'num'         => $num++,
				'uid'         => $row->domain,
				'domain'      => "<a href='{$row->domain}' target='_blank'>{$row->domain}</a>",
				'spam_active' => isset($row->spam_active) ? ($row->spam_active ? 'Yes' : 'No') : 'Unknown',
				'link_count'  => htmlspecialchars($row->link_count),
				'actions'     => $row->actions,
			);
		}
	}	
}

function spbc_field_scanner__prepare_data__links(&$table){
	if($table->items_count){
		foreach($table->rows as $row){
			$table->items[] = array(
				'link_id'     => $row->link_id,
				'link'        => "<a href='{$row->link}' target='_blank'>{$row->link}</a>",
				'page_url'    => "<a href='{$row->page_url}' target='_blank'>{$row->page_url}</a>",
				'link_text'   => htmlspecialchars($row->link_text),
				'spam_active' => isset($row->spam_active) ? ($row->spam_active ? 'Yes' : 'No') : 'Unknown',
			);
		}
	}	
}

function spbc_field_scaner($params = array()){
	
	global $spbc, $spbc_tpl, $wp_version;
	
	echo '<div class="spbc_wrapper_field">';
	
	if(!$spbc->key_is_ok){
		
		$button = '<input type="button" class="button button-primary" value="'.__('To setting', 'security-malware-firewall').'"  />';
		$link = sprintf('<a href="#" onclick="spbc_switchTab(document.getElementsByClassName(\'spbc_tab_nav-settings_general\')[0], \'spbc_key\', 3);">%s</a>', $button);
		echo '<div style="margin: 10px auto; text-align: center;"><h3 style="margin: 5px; display: inline-block;">'.__('Please, enter API key.', 'security-malware-firewall').'</h3>'.$link.'</div>';
		
	}elseif(!$spbc->scaner_status){
		
		$button = '<input type="button" class="button button-primary" value="'.__('RENEW', 'security-malware-firewall').'"  />';
		$link = sprintf('<a target="_blank" href="https://cleantalk.org/my/bill/security?cp_mode=security&utm_source=wp-backend&utm_medium=cpc&utm_campaign=WP%%20backend%%20trial_security&user_token=%s">%s</a>', $spbc->user_token, $button);
		echo '<div style="margin-top: 10px;"><h3 style="margin: 5px; display: inline-block;">'.__('Please renew your security license.', 'security-malware-firewall').'</h3>'.$link.'</div>';
		
	}else{
	
		if(preg_match('/^[\d\.]*$/', $wp_version) !== 1){
			echo '<p class="spbc_hint" style="text-align: center;">';
				printf(__('Your Wordpress version %s is not supported', 'security-malware-firewall'), $wp_version);
			echo '</p>';
			// return;
		}
		
		echo '<p class="spbc_hint" style="text-align: center;">';
		if(empty($spbc->data['scanner']['last_scan']))
			_e('System hasn\'t been scanned yet. Please, perform the scan to secure the website.', 'security-malware-firewall');
		elseif($spbc->data['scanner']['last_scan'] < time() - 86400 * 7){
			_e('System hasn\'t been scanned for a long time', 'security-malware-firewall');
		}
		else{
			_e('Look below for scan results.', 'security-malware-firewall');
		}
		echo '</br>';
		printf(
			__('%sView all scan results for this website%s', 'security-malware-firewall'),
			'<a target="blank" href="https://cleantalk.org/my/logs_mscan?service='.$spbc->service_id.'">',
			'</a>'
		);
		echo '</p>';
		
		echo '<div style="text-align: center;">'
			.'<button id="spbc_perform_scan" class="spbc_manual_link" type="button">'
				.__('Perform scan', 'security-malware-firewall')
			.'</button>'
			.'<img  class="spbc_preloader" src="'.SPBC_PATH.'/images/preloader.gif" />'
		.'</div>';
		
		echo '<p class="spbc_hint" style="text-align: center; margin-top: 5px;">';
			if(isset($spbc->data['scanner']['last_scan'])){
				printf(
					__('Website last scan was performed on %s, %d files were scanned.', 'security-malware-firewall'),
					date('M d Y H:i:s',$spbc->data['scanner']['last_scan']),
					$spbc->data['scanner']['last_scan_amount']
				);
				if($spbc->settings['scanner_outbound_links'])
					printf(' '.__('%s outbound links were found.', 'security-malware-firewall'), isset($spbc->data['scanner']['last_scan_links_amount']) ? $spbc->data['scanner']['last_scan_links_amount'] : 0);
			}else
				__('Website hasn\'t been scanned yet.', 'security-malware-firewall');
			if(isset($spbc->data['scanner']['last_scan'])){
				printf(
					' '.__('Next automatic scan scheduled to %s.', 'security-malware-firewall'),
					date('M d Y H:i:s', $spbc->data['scanner']['last_scan']+86352)
				);
			}
				
		echo '</p>';
					
		/* Debug Buttons
			echo '<button id="spbc_scanner_clear" class="spbc_manual_link" type="button">'
			.__('Clear', 'security-malware-firewall')
			.'</button>'
			.'<img class="spbc_preloader" src="http://wordpress.loc/wp-content/plugins/security-malware-firewall/images/preloader.gif" />'
			.'<br /><br />';
		//*/
		
		echo 
		'<div id="spbc_scaner_progress_overall" class="spbc_hide" style="padding-bottom: 10px; text-align: center;">'
			.'<span class="spbc_overall_scan_status_get_hashes">'       .__('Receiving core hashes', 'security-malware-firewall')              .'</span> -> '
			.'<span class="spbc_overall_scan_status_count_hashes_plug">'.__('Counting plugins and themes', 'security-malware-firewall')        .'</span> -> '
			.'<span class="spbc_overall_scan_status_get_hashes_plug">'  .__('Receiving plugins and themes hashes', 'security-malware-firewall').'</span> -> '
			.'<span class="spbc_overall_scan_status_clear_table">'      .__('Preparing', 'security-malware-firewall')                          .'</span> -> '
			.'<span class="spbc_overall_scan_status_count">'            .__('Counting files', 'security-malware-firewall')                     .'</span> -> '
			.'<span class="spbc_overall_scan_status_scan">'             .__('Scanning for modifications', 'security-malware-firewall')         .'</span> -> ';

			if($spbc->settings['scanner_signature_analysis'])
				echo '<span class="spbc_overall_scan_status_count_modified_sign">'.__('Counting not checked', 'security-malware-firewall').'</span> -> '
				.'<span class="spbc_overall_scan_status_scan_modified_sign">'.__('Signature analysis', 'security-malware-firewall').'</span> -> ';
			
			if($spbc->settings['scanner_heuristic_analysis'])
				echo '<span class="spbc_overall_scan_status_count_modified_heur">'.__('Count not checked', 'security-malware-firewall').'</span> -> '
				.'<span class="spbc_overall_scan_status_scan_modified_heur">'.__('Heuristic analysis', 'security-malware-firewall').'</span> -> ';
			
			if($spbc->settings['scanner_auto_cure'])
				echo '<span class="spbc_overall_scan_status_count_cure">'.__('Count cure', 'security-malware-firewall').'</span> -> '
				.'<span class="spbc_overall_scan_status_cure">'.__('Curing', 'security-malware-firewall').'</span> -> ';
			
			if($spbc->settings['scanner_outbound_links'])
				echo '<span class="spbc_overall_scan_status_count_links">'.__('Counting links', 'security-malware-firewall').'</span> -> '
				.'<span class="spbc_overall_scan_status_scan_links">'.__('Scanning links', 'security-malware-firewall').'</span> -> ';
			
			echo '<span class="spbc_overall_scan_status_send_results">'.__('Sending results', 'security-malware-firewall').'</span>'
			
		.'</div>';
		echo '<div id="spbc_scaner_progress_bar" class="spbc_hide" style="height: 22px;"><div class="spbc_progressbar_counter"><span></span></div></div>';
		
		echo '<div id="spbc_dialog" title="File output" style="overflow: initial;"></div>';
		echo '<div id="spbc_scan_accordion">';
			
			$tables_files = array(
				'critical'    => __('These files may not contain malicious code but they use very dangerous PHP functions and constructions! Using such doesn\'t recommend by PHP developers or looks very suspicious.', 'security-malware-firewall'),
				'suspicious'  => __('Executable files which has been modified.', 'security-malware-firewall'),
				'unknown'     => __('Unknown executable files spotted in core or plugins. These files don\'t come with wordpress distributive. It could be anything.', 'security-malware-firewall'),
				'approved'    => __('Approved files', 'security-malware-firewall'),
				'quarantined' => __('Punished files.', 'security-malware-firewall'),
			);
			
			if($spbc->settings['scanner_outbound_links'])
				$tables_files['outbound_links'] = __('Shows you the list of outgoing links from your website and websites on which they linking to.', 'security-malware-firewall');
			
			// Default arguments for file tables
			$default_args = array(
				'sql' => array(
					'add_col'     => array('fast_hash', 'last_sent', 'real_full_hash', 'severity', 'difference', 'status',),
					'table'       => SPBC_TBL_SCAN_FILES,
					'offset'      => 0,
					'limit'       => SPBC_LAST_ACTIONS_TO_VIEW,
					'get_array'  => false,
				),
				'if_empty_items' => 'NOPE',
				'columns' => array(
					'cb'       => array('heading' => '<input type=checkbox>',	'class' => 'check-column',),
					'path'     => array('heading' => 'Path','primary' => true,),
					'size'     => array('heading' => 'Size, bytes',),
					'perms'    => array('heading' => 'Permissions',),
					'mtime'    => array('heading' => 'Last Modified',),
				),
				'actions' => array(
					'delete'  => array('name' => 'Delete',),
					'view'    => array('name' => 'View', 'handler' => 'spbc_scanner_button_file_view_event(this);',),
				),
				'bulk_actions'  => array(
					'delete'  => array('name' => 'Delete',),
				),
				'sortable' => array('path', 'size', 'perms', 'mtime',),
				'pagination' => array(
					'page'     => 1,
					'per_page' => SPBC_LAST_ACTIONS_TO_VIEW,
				),
				'order_by'  => array('path' => 'asc'),
			);
			
			foreach($tables_files as $type_name => $description){
				
				if($type_name == 'critical'){
					$args = array_replace_recursive(
						$default_args,
						array(
							'columns' => array(
								'cb'         => array('heading' => '<input type=checkbox>',	'class' => 'check-column',),
								'path'       => array('heading' => 'Path','primary' => true,),
								'size'       => array('heading' => 'Size, bytes',),
								'perms'      => array('heading' => 'Permissions',),
								'weak_spots' => array('heading' => 'Detected'),
								'mtime'      => array('heading' => 'Last Modified',),
							),
							'func_data_prepare' => 'spbc_field_scanner__prepare_data__files',
							'if_empty_items' => '<p class="spbc_hint">'.__('No threats are found', 'security-malware-firewall').'</p>',
							'actions' => array(
								'send'       => array('name' => 'Send for Analysis',),
								'approve'    => array('name' => 'Approve',),
								'quarantine' => array('name' => 'Quarantine it',),
								'replace'    => array('name' => 'Replace with Original',),
								'delete'     => array('name' => 'Delete',),
								'compare'    => array('name' => 'Compare',       'handler' => 'spbc_scanner_button_file_compare_event(this);',),
								'view'       => array('name' => 'View',          'handler' => 'spbc_scanner_button_file_view_event(this);',),
								'view_bad'   => array('name' => 'View Bad Code', 'handler' => 'spbc_scanner_button_file_view_bad_event(this);',),
							),
							'bulk_actions'  => array(
								'send'       => array('name' => 'Send',),
								'approve'    => array('name' => 'Approve',),
								'delete'     => array('name' => 'Delete',),
								'replace'    => array('name' => 'Replace with original',),
								'quarantine' => array('name' => 'Quarantine it',),
							),
							'sql' => array(
//								'where' => ' WHERE severity IN("CRITICAL", "DANGER", "SUSPICIOUS") AND status <> "QUARANTINED"',
								'where' => ' WHERE severity IN("CRITICAL") AND status <> "QUARANTINED"',
							)
						)
					);
				}elseif($type_name == 'suspicious'){
					$args = array_replace_recursive(
						$default_args,
						array(
							'func_data_prepare' => 'spbc_field_scanner__prepare_data__files',
							'order_by'  => array('path' => 'asc'),
							'if_empty_items' => false,
							'actions' => array(
								'send'       => array('name' => 'Send for Analysis',),
								'approve'    => array('name' => 'Approve',),
								'quarantine' => array('name' => 'Quarantine it',),
								'replace'    => array('name' => 'Replace with Original',),
								'delete'     => array('name' => 'Delete',),
								'compare'    => array('name' => 'Compare',       'handler' => 'spbc_scanner_button_file_compare_event(this);',),
								'view'       => array('name' => 'View',          'handler' => 'spbc_scanner_button_file_view_event(this);',),
								'view_bad'   => array('name' => 'View Bad Code', 'handler' => 'spbc_scanner_button_file_view_bad_event(this);',),
							),
							'bulk_actions'  => array(
								'send'       => array('name' => 'Send',),
								'approve'    => array('name' => 'Approve',),
								'delete'     => array('name' => 'Delete',),
								'replace'    => array('name' => 'Replace with original',),
								'quarantine' => array('name' => 'Quarantine it',),
							),
							'sql' => array(
								'where' => ' WHERE status = "MODIFIED" AND severity <> "CRITICAL"',
							)
						)
					);
				}elseif($type_name == 'unknown'){
					$args = array_replace_recursive(
						$default_args,
						array(
							'func_data_prepare' => 'spbc_field_scanner__prepare_data__files',
							'if_empty_items' => false,
							'actions' => array(
								'approve' => array('name' => 'Approve',),
							),
							'bulk_actions'  => array(
								'approve' => array('name' => 'Approve',),
							),
							'sql' => array(
								'where' => ' WHERE status = "UNKNOWN" AND severity <> "CRITICAL"',
							)
						)
					);
				}elseif($type_name == 'approved'){
					$args = array_replace_recursive(
						$default_args,
						array(
							'func_data_prepare' => 'spbc_field_scanner__prepare_data__files',
							'if_empty_items' => false,
							'actions' => array(
								'disapprove' => array('name' => 'Disapprove',),
							),
							'bulk_actions'  => array(
								'disapprove' => array('name' => 'Disapprove',),
							),
							'sql' => array(
								'where' => ' WHERE status = "APROVED" AND severity <> "CRITICAL"',
							)
						)
					);
				}elseif($type_name == 'quarantined'){
					$args = array_replace_recursive(
						$default_args,
						array(
							'func_data_prepare' => 'spbc_field_scanner__prepare_data__files_qurantine',
							'columns' => array(
								'cb'       => array('heading' => '<input type=checkbox>',	'class' => 'check-column',),
								'path'     => array('heading' => 'Path','primary' => true,),
								'q_status' => array('heading' => 'Status',),
								'severity' => array('heading' => 'Severity',),
								'q_time'   => array('heading' => 'Quarantine time',),
								'size'     => array('heading' => 'Size',),
							),
							'if_empty_items' => false,
							'actions' => array(
								'restore'  => array('name' => 'Restore',),
								'delete'   => array('name' => 'Delete',),
								'view'     => array('name' => 'View', 'handler' => 'spbc_scanner__show_link_table();',),
								'download' => array(
									'name'   => 'Download',
									'type'   => 'link',
									'local'  => true,
									'uid'    => true,
									'target' => '_blank',
									'href'   => '?plugin_name=security&spbc_remote_call_token='.md5($spbc->settings['spbc_key']).'&spbc_remote_call_action=q_download&file_id=',
								),
							),
							'bulk_actions'  =>  array(
								'restore'    => array('name' => 'Restore',),
								'delete'     => array('name' => 'Delete',),
							),
							'sql' => array(
								'add_col' => array_merge($args['sql']['add_col'], array('q_status', 'q_path', 'q_time',)),
								'where' => ' WHERE status = "QUARANTINED"',
							),
							'sortable' => array('path', 'q_status', 'severity', 'q_time', 'size',),
						)
					);
				}elseif($type_name == 'outbound_links'){
					$args = array(	
						'id' => 'spbc_tbl__scanner_outbound_links',
						'actions' => array(
							'show_links' => array(
								'name' => 'Show links',
								'handler' => 'spbc_scanner__switch_table(this, "links");'
							),
						),
						'order_by'  => array('spam_active' => 'desc'),
						'func_data_total'   => 'spbc_scanner_links_count_found__domains',
						'func_data_get'     => 'spbc_scanner_links_get_scanned__domains',
						'func_data_prepare' => 'spbc_field_scanner__prepare_data__domains',
						'if_empty_items' => '<p class="spbc_hint">'.__('No links are found', 'security-malware-firewall').'</p>',
						'columns' => array(
							'num'         => array('heading' => __('Number', 'security-malware-firewall'), 'class' => ' tbl-width--50px'),
							'domain'      => array('heading' => __('Domain', 'security-malware-firewall'), 'primary' => true,),
							'spam_active' => array(
								'heading' => __('Spam-active', 'security-malware-firewall'),
								'hint' => __('Does link spotted in spam?', 'security-malware-firewall'),),
							'link_count'  => array(
								'heading' => __('Links of domain', 'security-malware-firewall'),
								'hint' => __('Number of found links to the domain on site.', 'security-malware-firewall'),),  
						),
						'sortable' => array('spam_active', 'domain', 'link_count'),
						'pagination' => array(
							'page'     => 1,
							'per_page' => SPBC_LAST_ACTIONS_TO_VIEW,
						),
					);
				}
				
				$args['id'] = 'spbc_tbl__scanner_'.$type_name;
				
				$table = new SpbcListTable($args);
				$table->get_data();
				
				// Pass output if empty and said to do so
				if($args['if_empty_items'] !== false || $table->items_total !== 0){
					
					echo '<h3><a href="#">'.ucwords(str_replace('_', ' ', $type_name)).' (<span class="spbc_bad_type_count '.$type_name.'_counter">'.$table->items_total.'</span>)</a></h3>';
					echo '<div id="spbc_scan_accordion_tab_'.$type_name.'">';

						echo '<p class="spbc_hint spbc_hint_warning	">'.$description.'</p>';
						$table->display();

					echo "</div>";
					
				}
				
			}
			
		echo '</div>';
	}
	echo '</div>';
}

function spbc_field_debug_drop(){
	echo '<div class="spbc_wrapper_field">'
		.'<br>'
		.'<input form="drop_debug" type="submit" name="spbc_drop_debug" value="Drop debug data" />'
		.'<div class="spbc_settings_description">If you don\'t what is this just push the button =)</div>'
	.'</div>';
}

function spbc_field_debug(){
	global $spbc;
	$output = print_r($spbc->debug, true);
	$output = str_replace("\n", "<br>", $output);
	$output = preg_replace("/[^\S]{4}/", "&nbsp;&nbsp;&nbsp;&nbsp;", $output);
	echo "<div class='spbc_wrapper_field'>";
		echo $output
		."<label for=''>".
			
		"</label>".
		"<div class='spbc_settings_description'>".
			
		"</div>";
	echo "</div>";
}

/**
 * Admin callback function - Sanitize settings
 */
function spbc_sanitize_settings( $settings ){
	
	global $spbc;
	
	//Sanitizing traffic_control_autoblock_amount setting
	if(isset($settings['traffic_control_autoblock_amount'])){
		$settings['traffic_control_autoblock_amount'] = floor(intval($settings['traffic_control_autoblock_amount']));
		$settings['traffic_control_autoblock_amount'] = ($settings['traffic_control_autoblock_amount'] == 0  ? 1000 : $settings['traffic_control_autoblock_amount']);
		$settings['traffic_control_autoblock_amount'] = ($settings['traffic_control_autoblock_amount'] <  20 ? 20   : $settings['traffic_control_autoblock_amount']);
	}
	
	// Send logs for 2 previous days
	if(isset($settings['backend_logs_enable']) && $settings['backend_logs_enable'] && (!isset($spbc->settings['backend_logs_enable']) || (isset($spbc->settings['backend_logs_enable']) && !$spbc->settings['backend_logs_enable']))){
		$spbc->data['last_php_log_sent'] = time()-86400*2;
		$spbc->save('data');
	}
	
	if(isset($settings['scanner_outbound_links_mirrors'])){
		if(preg_match('/^[\sa-zA-Z0-9,_\.\-\~]+$/', $settings['scanner_outbound_links_mirrors'])){
			$tmp = explode(',', $settings['scanner_outbound_links_mirrors']);
			foreach($tmp as $key => $value){
				$value = trim($value);
				if(!empty($value))
					$mirrors[$key] = trim($value);
			} unset ($key, $value);
			$settings['scanner_outbound_links_mirrors'] = implode(', ', $mirrors);
		}
	}
	
	//Checking the accepted key
	$settings['spbc_key'] = isset($settings['spbc_key']) 
		? trim($settings['spbc_key'])
		: (isset($spbc->network_settings['spbc_key'])
			? $spbc->network_settings['spbc_key']
			: '');
			
	if(strpos($settings['spbc_key'], '*') !== false && $settings['spbc_key'] == str_repeat('*', strlen($spbc->settings['spbc_key']))){
		$settings['spbc_key'] = $spbc->settings['spbc_key'];
	}
	preg_match('/^[a-z\d]*$/', $settings['spbc_key'], $matches);
	$sanitized_key = !empty($matches[0]) ? $matches[0] : '';
	
	if($sanitized_key == ''){
		$spbc->data['key_is_ok']          = false;
		$spbc->data['notice_show']        = 0;
		$spbc->data['notice_renew']       = 0;
		$spbc->data['notice_trial']       = 0;
		$spbc->data['notice_auto_update'] = 0;
		$spbc->data['user_token']         = 0;
		$spbc->data['service_id']         = '';
		$spbc->data['moderate']	          = 0;
		$spbc->data['auto_update_app']    = 0;
		$spbc->data['license_trial']      = 0;
		$spbc->data['account_name_ob']    = '';
		$spbc->error_delete(array(
			'notice_show',
			'notice_renew',
			'notice_trial',
			'notice_were_updated',
			'user_token',
			'service_id',
		));
		$spbc->error_add('apikey', __('Key is empty.', 'security-malware-firewall'));
	}else{
		
		//Clearing all errors
		$spbc->error_delete_all('and_save_data');		
		$result = SpbcHelper::api_method__notice_validate_key($sanitized_key, preg_replace('/http[s]?:\/\//', '', get_option('siteurl'), 1));
		
		if(empty($result['error'])){
				
			if($result['valid'] == '1' ){
				$spbc->data['key_is_ok'] = true;
			}else{
				$spbc->data['key_is_ok']          = false;
				$spbc->data['notice_show']        = 0;
				$spbc->data['notice_renew']       = 0;
				$spbc->data['notice_trial']       = 0;
				$spbc->data['notice_auto_update'] = 0;
				$spbc->data['user_token']         = 0;
				$spbc->data['service_id']         = '';
				$spbc->data['moderate']	          = 0;
				$spbc->data['auto_update_app']    = 0;
				$spbc->data['license_trial']      = 0;
				$spbc->data['account_name_ob']    = '';
				$spbc->error_delete(array(
					'notice_show',
					'notice_renew',
					'notice_trial',
					'notice_were_updated',
					'user_token',
					'service_id',
				));
				$spbc->error_add('apikey', sprintf(__('Key is not valid. Key: %s.', 'security-malware-firewall'), $sanitized_key));
			}
			
		}else{
			$spbc->error_add('apikey', $result);
		}
	}
	
	// If key is ok
	if($spbc->data['key_is_ok'] == true){
		
		// Sending logs.
		$result = spbc_send_logs($sanitized_key);		
		if(empty($result['error'])){
			$spbc->data['logs_last_sent'] = current_time('timestamp');
			$spbc->data['last_sent_events_count'] = $result;
			$spbc->error_delete('send_logs');
		}else{
			$spbc->error_add('send_logs', $result);
		}
		
		// Updating FW
		$result = spbc_security_firewall_update($sanitized_key, true);
		if(empty($result['error'])){
			$spbc->data['last_firewall_updated'] = current_time('timestamp');
			$spbc->data['firewall_entries']      = $result;
			$spbc->error_delete('firewall_update');
		}else{
			$spbc->error_add('firewall_update', $result);
		}
		
		// Sending FW logs
		$result = spbc_send_firewall_logs($sanitized_key);
		if(empty($result['error'])){
			$spbc->data['last_firewall_send'] = current_time('timestamp');
			$spbc->data['last_firewall_send_count'] = $result;
			$spbc->error_delete('send_firewall_logs');
		}else{
			$spbc->error_add('send_firewall_logs', $result);
		}
		
		// Checking account status
		$result = SpbcHelper::api_method__notice_paid_till($sanitized_key);
		if(empty($result['error'])){
			if(isset($result['user_token'])) $spbc->data['user_token'] = $result['user_token'];
			$spbc->data['notice_show']	    = $result['show_notice'];
			$spbc->data['notice_renew']     = $result['renew'];
			$spbc->data['notice_trial']     = $result['trial'];
			$spbc->data['auto_update_app']  = isset($result['show_auto_update_notice']) ? $result['show_auto_update_notice'] : 0;
			$spbc->data['service_id']       = $result['service_id'];
			$spbc->data['moderate']	        = $result['moderate'];
			$spbc->data['auto_update_app '] = isset($result['auto_update_app']) ? $result['auto_update_app'] : 0;
			$spbc->data['license_trial']    = isset($result['license_trial']) ? $result['license_trial'] : 0;
			$spbc->data['account_name_ob']  = isset($result['account_name_ob']) ? $result['account_name_ob'] : '';
			$spbc->error_delete('access_key_notices');
		}else{
			$spbc->error_add('access_key_notices', $result);
		}		
	}
	
	$spbc->save('data');
	
	$settings['spbc_key'] = $sanitized_key;
	
	if(SPBC_WPMS && is_main_site()){
			
		$spbc->network_settings = array(
			'key_is_ok'          => $spbc->data['key_is_ok'],
			'spbc_key'           => $settings['spbc_key'],
			'user_token'         => isset($spbc->data['user_token']) ? $spbc->data['user_token'] : '',
			'allow_custom_key'   => isset($settings['custom_key'])   ? $settings['custom_key']   : false,
			'allow_cleantalk_cp' => isset($settings['allow_ct_cp'])  ? $settings['allow_ct_cp']  : false,
			'service_id'         => isset($spbc->data['service_id']) ? $spbc->data['service_id'] : ''
		);
		$spbc->saveNetworkSettings();
	}
	
	$settings = array(
		'spbc_key'                         => $settings['spbc_key'],
		'traffic_control_enabled'          => !empty($settings['traffic_control_enabled'])          ? $settings['traffic_control_enabled']          : false,
		'custom_key'                       => !empty($settings['custom_key'])                       ? $settings['custom_key']                       : false,
		'traffic_control_autoblock_amount' => !empty($settings['traffic_control_autoblock_amount']) ? $settings['traffic_control_autoblock_amount'] : '1000',
		'show_link_in_login_form'          => !empty($settings['show_link_in_login_form'])          ? $settings['show_link_in_login_form']          : false,
		'set_cookies'                      => !empty($settings['set_cookies'])                      ? $settings['set_cookies']                      : false,
		'2fa_enable'                       => !empty($settings['2fa_enable'])                       ? $settings['2fa_enable']                       : false,
		'complete_deactivation'            => !empty($settings['complete_deactivation'])            ? $settings['complete_deactivation']            : false,
		'scanner_outbound_links'		   => !empty($settings['scanner_outbound_links'])           ? $settings['scanner_outbound_links']           : false,
		'scanner_heuristic_analysis'	   => !empty($settings['scanner_heuristic_analysis'])       ? $settings['scanner_heuristic_analysis']       : false,
		'scanner_signature_analysis'	   => !empty($settings['scanner_signature_analysis'])       ? $settings['scanner_signature_analysis']       : false,
		'scanner_auto_cure'          	   => !empty($settings['scanner_auto_cure'])                ? $settings['scanner_auto_cure']                : false,
		'scanner_outbound_links_mirrors'   => !empty($settings['scanner_outbound_links_mirrors'])   ? $settings['scanner_outbound_links_mirrors']   : '',
		'backend_logs_enable'              => !empty($settings['backend_logs_enable'])              ? $settings['backend_logs_enable']              : false,
		'waf_enabled'                      => !empty($settings['waf_enabled'])                      ? $settings['waf_enabled']                      : false,
		'waf_xss_check'                    => !empty($settings['waf_xss_check'])                    ? $settings['waf_xss_check']                    : false,
		'waf_sql_check'                    => !empty($settings['waf_sql_check'])                    ? $settings['waf_sql_check']                    : false,
		'waf_file_check'                   => !empty($settings['waf_file_check'])                   ? $settings['waf_file_check']                   : false,
	);
	
	return $settings;
}


function spbc_show_more_security_logs_callback(){
	
	check_ajax_referer('spbc_secret_nonce', 'security');
	
	// PREPROCESS INPUT
	$args = $_POST['args'];
	$amount  = isset($_POST['amount']) ? intval($_POST['amount']) : SPBC_LAST_ACTIONS_TO_VIEW;
	
	$args['pagination'] = array();
	$args['sql']['limit'] = $amount;
	
	// OUTPUT
	$table = new SpbcListTable($args);
	
	$table->get_data();
	
	die(
		json_encode(
			array(
				'html' => $table->display__rows('return'),
				'size' => $table->items_count,
			)
		)
	);
}

function spbc_show_more_security_firewall_logs_callback(){
	
	check_ajax_referer('spbc_secret_nonce', 'security');
	
	// PREPROCESS INPUT
	$args = $_POST['args'];
	$amount  = isset($_POST['amount']) ? intval($_POST['amount']) : SPBC_LAST_ACTIONS_TO_VIEW;
	
	$args['pagination'] = array();
	$args['sql']['limit'] = $amount;
	
	// OUTPUT
	$table = new SpbcListTable($args);
	
	$table->get_data();
	
	if($_POST['full_refresh'])
		die($table->display());
	
	die(
		json_encode(
			array(
				'html' => $table->display__rows('return'),
				'size' => $table->items_count,
			)
		)
	);
}

function spbc_show_GDPR_text(){

	return wpautop('The notice requirements remain and are expanded. They must include the retention time for personal data, and contact information for data controller and data protection officer has to be provided.
	Automated individual decision-making, including profiling (Article 22) is contestable, similarly to the Data Protection Directive (Article 15). Citizens have rights to question and fight significant decisions that affect them that have been made on a solely-algorithmic basis. Many media outlets have commented on the introduction of a "right to explanation" of algorithmic decisions, but legal scholars have since argued that the existence of such a right is highly unclear without judicial tests and is limited at best.
	To be able to demonstrate compliance with the GDPR, the data controller should implement measures, which meet the principles of data protection by design and data protection by default. Privacy by design and by default (Article 25) require data protection measures to be designed into the development of business processes for products and services. Such measures include pseudonymising personal data, by the controller, as soon as possible (Recital 78).
	It is the responsibility and the liability of the data controller to implement effective measures and be able to demonstrate the compliance of processing activities even if the processing is carried out by a data processor on behalf of the controller (Recital 74).
	Data Protection Impact Assessments (Article 35) have to be conducted when specific risks occur to the rights and freedoms of data subjects. Risk assessment and mitigation is required and prior approval of the national data protection authorities (DPAs) is required for high risks. Data protection officers (Articles 37–39) are required to ensure compliance within organisations.
	They have to be appointed:')
	.'<ul style="padding: 0px 25px; list-style: disc;">'
		.'<li>for all public authorities, except for courts acting in their judicial capacity</li>'
		.'<li>if the core activities of the controller or the processor are:</li>'
			.'<ul style="padding: 0px 25px; list-style: disc;">'
				.'<li>processing operations, which, by virtue of their nature, their scope and/or their purposes, require regular and systematic monitoring of data subjects on a large scale</li>'
				.'<li>processing on a large scale of special categories of data pursuant to Article 9 and personal data relating to criminal convictions and offences referred to in Article 10;</li>'
			.'</ul>'
		.'</li>'
	.'</ul>';
}